@@ -28,7 +28,7 @@ require_once 'Zend/Validate/Interface.php';
  * @package    Zend_Form
  * @copyright  Copyright (c) 2005-2008 Zend Technologies USA Inc. (http://www.zend.com)
  * @license    http://framework.zend.com/license/new-bsd     New BSD License
- * @version    $Id: Form.php 8996 2008-03-22 16:59:06Z matthew $
+ * @version    $Id: Form.php 10372 2008-07-24 16:22:58Z matthew $
  */
 class Zend_Form implements Iterator, Countable, Zend_Validate_Interface
 {
@@ -239,6 +239,11 @@ class Zend_Form implements Iterator, Countable, Zend_Validate_Interface
             unset($options['elementPrefixPath']);                             
         }
 
+        if (isset($options['displayGroupPrefixPath'])) {                          
+            $this->addDisplayGroupPrefixPaths($options['displayGroupPrefixPath']);      
+            unset($options['elementPrefixPath']);                             
+        }
+
         if (isset($options['elements'])) {
             $this->setElements($options['elements']);
             unset($options['elements']);
@@ -473,6 +478,10 @@ class Zend_Form implements Iterator, Countable, Zend_Validate_Interface
             $element->addPrefixPath($prefix, $path, $type);
         }
 
+        foreach ($this->getSubForms() as $subForm) {
+            $subForm->addElementPrefixPath($prefix, $path, $type);
+        }
+
         return $this;
     }
 
@@ -502,7 +511,10 @@ class Zend_Form implements Iterator, Countable, Zend_Validate_Interface
      */
     public function addDisplayGroupPrefixPath($prefix, $path)
     {
-        $this->_displayGroupPrefixPaths[] = array($prefix, $path);
+        $this->_displayGroupPrefixPaths[] = array(
+            'prefix' => $prefix, 
+            'path'   => $path,
+        );
 
         foreach ($this->getDisplayGroups() as $group) {
             $group->addPrefixPath($prefix, $path);
@@ -511,6 +523,43 @@ class Zend_Form implements Iterator, Countable, Zend_Validate_Interface
         return $this;
     }
 
+    /**
+     * Add multiple display group prefix paths at once
+     * 
+     * @param  array $spec 
+     * @return Zend_Form
+     */
+    public function addDisplayGroupPrefixPaths(array $spec)
+    {
+        foreach ($spec as $key => $value) {
+            if (is_string($value) && !is_numeric($key)) {
+                $this->addDisplayGroupPrefixPath($key, $value);
+                continue;
+            }
+
+            if (is_string($value) && is_numeric($key)) {
+                continue;
+            }
+
+            if (is_array($value)) {
+                $count = count($value);
+                if (array_keys($value) === range(0, $count - 1)) {
+                    if ($count < 2) {
+                        continue;
+                    }
+                    $prefix = array_shift($value);
+                    $path   = array_shift($value);
+                    $this->addDisplayGroupPrefixPath($prefix, $path);
+                    continue;
+                }
+                if (array_key_exists('prefix', $value) && array_key_exists('path', $value)) {
+                    $this->addDisplayGroupPrefixPath($value['prefix'], $value['path']);
+                }
+            }
+        }
+        return $this;
+    }
+
     // Form metadata:
     
     /**
@@ -713,6 +762,47 @@ class Zend_Form implements Iterator, Countable, Zend_Validate_Interface
         return $this->getAttrib('name');
     }
 
+    /**
+     * Get fully qualified name
+     *
+     * Places name as subitem of array and/or appends brackets.
+     * 
+     * @return string
+     */
+    public function getFullyQualifiedName()
+    {
+        return $this->getName();
+    }
+
+    /**
+     * Get element id
+     * 
+     * @return string
+     */
+    public function getId()
+    {
+        if (null !== ($id = $this->getAttrib('id'))) {
+            return $id;
+        }
+
+        $id = $this->getFullyQualifiedName();
+
+        // Bail early if no array notation detected
+        if (!strstr($id, '[')) {
+            return $id;
+        }
+
+        // Strip array notation
+        if ('[]' == substr($id, -2)) {
+            $id = substr($id, 0, strlen($id) - 2);
+        }
+        $id = str_replace('][', '-', $id);
+        $id = str_replace(array(']', '['), '-', $id);
+        $id = trim($id, '-');
+
+        return $id;
+    }
+
     /**
      * Set form legend
      * 
@@ -823,6 +913,7 @@ class Zend_Form implements Iterator, Countable, Zend_Validate_Interface
 
         $this->_order[$name] = $this->_elements[$name]->getOrder();
         $this->_orderUpdated = true;
+        $this->_setElementsBelongTo($name);
 
         return $this;
     }
@@ -954,7 +1045,7 @@ class Zend_Form implements Iterator, Countable, Zend_Validate_Interface
      */
     public function getElement($name)
     {
-        if (isset($this->_elements[$name])) {
+        if (array_key_exists($name, $this->_elements)) {
             return $this->_elements[$name];
         }
         return null;
@@ -1103,18 +1194,12 @@ class Zend_Form implements Iterator, Countable, Zend_Validate_Interface
             }
         }
         foreach ($this->getSubForms() as $key => $subForm) {
-            $array = $this->_getArrayName($subForm->getElementsBelongTo());
-            if (empty($array)) {
-                $values[$key] = $subForm->getValues(true);
-            } else {
-                $values[$array] = $subForm->getValues(true);
-            }
+            $fValues = $this->_attachToArray($subForm->getValues(true), $subForm->getElementsBelongTo());
+            $values = array_merge($values, $fValues);
         }
 
         if (!$suppressArrayNotation && $this->isArray()) {
-            $values = array(
-                $this->getElementsBelongTo() => $values
-            );
+            $values = $this->_attachToArray($values, $this->getElementsBelongTo());
         }
 
         return $values;
@@ -1171,19 +1256,51 @@ class Zend_Form implements Iterator, Countable, Zend_Validate_Interface
      */
     public function setElementsBelongTo($array)
     {
+        $origName = $this->getElementsBelongTo();
         $name = $this->filterName($array, true);
         if (empty($name)) {
             $name = null;
         }
         $this->_elementsBelongTo = $name;
 
-        (null !== $name) 
-            ? $this->setIsArray(true) 
-            : $this->setIsArray(false);
+        if (null === $name) {
+            $this->setIsArray(false);
+            if (null !== $origName) {
+                $this->_setElementsBelongTo();
+            }
+        } else {
+            $this->setIsArray(true);
+            $this->_setElementsBelongTo();
+        }
 
         return $this;
     }
 
+    /**
+     * Set array to which elements belong
+     * 
+     * @param  string $name Element name
+     * @return void
+     */
+    protected function _setElementsBelongTo($name = null)
+    {
+        $array = $this->getElementsBelongTo();
+
+        if (null === $array) {
+            return;
+        }
+
+        if (null === $name) {
+            foreach ($this->getElements() as $element) {
+                $element->setBelongsTo($array);
+            }
+        } else {
+            if (null !== ($element = $this->getElement($name))) {
+                $element->setBelongsTo($array);
+            }
+        }
+    }
+
     /**
      * Get name of array elements belong to
      * 
@@ -1204,7 +1321,6 @@ class Zend_Form implements Iterator, Countable, Zend_Validate_Interface
      * Set flag indicating elements belong to array
      * 
      * @param  bool $flag Value of flag
-     * @param  bool $setName Whether or not to set the name the elements belong to
      * @return Zend_Form
      */
     public function setIsArray($flag)
@@ -1669,6 +1785,65 @@ class Zend_Form implements Iterator, Countable, Zend_Validate_Interface
         $name = substr($value, $start, $endPos - $start);
         return $name;
     }
+    
+    /**
+     * Extract the value by walking the array using given array path.
+     *
+     * Given an array path such as foo[bar][baz], returns the value of the last
+     * element (in this case, 'baz').
+     * 
+     * @param  array $value Array to walk
+     * @param  string $arrayPath Array notation path of the part to extract
+     * @return string
+     */
+    protected function _dissolveArrayValue($value, $arrayPath)
+    {
+        // As long as we have more levels
+        while ($arrayPos = strpos($arrayPath, '[')) {
+            // Get the next key in the path
+            $arrayKey = trim(substr($arrayPath, 0, $arrayPos), ']');
+
+            // Set the potentially final value or the next search point in the array
+            if (isset($value[$arrayKey])) {
+                $value = $value[$arrayKey];
+            }
+            
+            // Set the next search point in the path
+            $arrayPath = trim(substr($arrayPath, $arrayPos + 1), ']');
+        }
+
+        if (isset($value[$arrayPath])) {
+            $value = $value[$arrayPath];
+        }
+
+        return $value;
+    }
+
+    /**
+     * Converts given arrayPath to an array and attaches given value at the end of it.
+     *
+     * @param  mixed $value The value to attach
+     * @param  string $arrayPath Given array path to convert and attach to.
+     * @return array 
+     */
+    protected function _attachToArray($value, $arrayPath)
+    {
+        // As long as we have more levels
+        while ($arrayPos = strrpos($arrayPath, '[')) {
+            // Get the next key in the path
+            $arrayKey = trim(substr($arrayPath, $arrayPos + 1), ']');
+
+            // Attach
+            $value = array($arrayKey => $value);
+            
+            // Set the next search point in the path
+            $arrayPath = trim(substr($arrayPath, 0, $arrayPos), ']');
+        }
+
+        $value = array($arrayPath => $value);
+
+        return $value;
+    }
 
     /**
      * Validate the form
@@ -1686,10 +1861,7 @@ class Zend_Form implements Iterator, Countable, Zend_Validate_Interface
         $valid      = true;
 
         if ($this->isArray()) {
-            $key = $this->_getArrayName($this->getElementsBelongTo());
-            if (isset($data[$key])) {
-                $data = $data[$key];
-            }
+            $data = $this->_dissolveArrayValue($data, $this->getElementsBelongTo());
         }
 
         foreach ($this->getElements() as $key => $element) {
@@ -1705,12 +1877,7 @@ class Zend_Form implements Iterator, Countable, Zend_Validate_Interface
             if (isset($data[$key])) {
                 $valid = $form->isValid($data[$key]) && $valid;
             } else {
-                $array = $this->_getArrayName($form->getElementsBelongTo());
-                if (empty($array) || !isset($data[$array])) {
-                    $valid = $form->isValid($data) && $valid;
-                } else {
-                    $valid = $form->isValid($data[$array]) && $valid;
-                }
+                $valid = $form->isValid($data) && $valid;
             }
         }
 
@@ -1729,10 +1896,7 @@ class Zend_Form implements Iterator, Countable, Zend_Validate_Interface
     public function isValidPartial(array $data)
     {
         if ($this->isArray()) {
-            $key = $this->_getArrayName($this->getElementsBelongTo());
-            if (isset($data[$key])) {
-                $data = $data[$key];
-            }
+            $data = $this->_dissolveArrayValue($data, $this->getElementsBelongTo());
         }
 
         $translator        = $this->getTranslator();
@@ -1758,12 +1922,8 @@ class Zend_Form implements Iterator, Countable, Zend_Validate_Interface
                 if (null !== $translator) {
                     $subForm->setTranslator($translator);
                 }
-                $array = $this->_getArrayName($subForm->getElementsBelongTo());
-                if (empty($array) || !isset($data[$array])) {
-                    $valid = $subForm->isValidPartial($data) && $valid;
-                } else {
-                    $valid = $subForm->isValidPartial($data[$array]) && $valid;
-                }
+
+                $valid = $subForm->isValidPartial($data) && $valid;
             }
         }
 
@@ -1823,12 +1983,8 @@ class Zend_Form implements Iterator, Countable, Zend_Validate_Interface
                 $errors[$key] = $element->getErrors();
             }
             foreach ($this->getSubForms() as $key => $subForm) {
-                $array = $this->_getArrayName($subForm->getElementsBelongTo());
-                if (empty($array) || !isset($data[$array])) {
-                    $errors[$key] = $subForm->getErrors();
-                } else {
-                    $errors[$array] = $subForm->getErrors();
-                }
+                $fErrors = $this->_attachToArray($subForm->getErrors(), $subForm->getElementsBelongTo());
+                $errors = array_merge($errors, $fErrors);
             }
         }
         return $errors;
@@ -1858,7 +2014,7 @@ class Zend_Form implements Iterator, Countable, Zend_Validate_Interface
                 if ($name == $array) {
                     return $subForm->getMessages(null, true);
                 }
-                $arrayKeys[$key] = $array;
+                $arrayKeys[$key] = $subForm->getElementsBelongTo();
             }
         }
 
@@ -1875,7 +2031,8 @@ class Zend_Form implements Iterator, Countable, Zend_Validate_Interface
             $fMessages = $subForm->getMessages(null, true);
             if (!empty($fMessages)) {
                 if (array_key_exists($key, $arrayKeys)) {
-                    $messages[$arrayKeys[$key]] = $fMessages;
+                    $fMessages = $this->_attachToArray($fMessages, $arrayKeys[$key]);
+                    $messages = array_merge($messages, $fMessages);
                 } else {
                     $messages[$key] = $fMessages;
                 }
@@ -1883,9 +2040,7 @@ class Zend_Form implements Iterator, Countable, Zend_Validate_Interface
         }
 
         if (!$suppressArrayNotation && $this->isArray()) {
-            $messages = array(
-                $this->getElementsBelongTo() => $messages
-            );
+            $messages = $this->_attachToArray($messages, $this->getElementsBelongTo());
         }
 
         return $messages;
@@ -2079,8 +2234,15 @@ class Zend_Form implements Iterator, Countable, Zend_Validate_Interface
     {
         $decorator = $this->getDecorator($name);
         if ($decorator) {
-            $name = get_class($decorator);
-            unset($this->_decorators[$name]);
+            if (array_key_exists($name, $this->_decorators)) {
+                unset($this->_decorators[$name]);
+            } else {
+                $class = get_class($decorator);
+                if (!array_key_exists($class, $this->_decorators)) {
+                    return false;
+                }
+                unset($this->_decorators[$class]);
+            }
             return true;
         }
 
@@ -2504,7 +2666,7 @@ class Zend_Form implements Iterator, Countable, Zend_Validate_Interface
             foreach ($this->_order as $key => $order) {
                 if (null === $order) {
                     if (null === ($order = $this->{$key}->getOrder())) {
-                        if (array_search($index, $this->_order, true)) {
+                        while (array_search($index, $this->_order, true)) {
                             ++$index;
                         }
                         $items[$index] = $key;
