@@ -14,57 +14,79 @@
  *
  * @category   Zend
  * @package    Zend_Translate
+ * @subpackage Zend_Translate_Adapter
  * @copyright  Copyright (c) 2005-2008 Zend Technologies USA Inc. (http://www.zend.com)
- * @version    $Id: Date.php 2498 2006-12-23 22:13:38Z thomas $
  * @license    http://framework.zend.com/license/new-bsd     New BSD License
+ * @version    $Id: Date.php 2498 2006-12-23 22:13:38Z thomas $
  */
 
-
-/** Zend_Locale */
+/**
+ * @see Zend_Locale
+ */
 require_once 'Zend/Locale.php';
 
 /**
+ * Basic adapter class for each translation source adapter
+ *
  * @category   Zend
  * @package    Zend_Translate
+ * @subpackage Zend_Translate_Adapter
  * @copyright  Copyright (c) 2005-2008 Zend Technologies USA Inc. (http://www.zend.com)
  * @license    http://framework.zend.com/license/new-bsd     New BSD License
  */
 abstract class Zend_Translate_Adapter {
+    /**
+     * Shows if locale detection is in automatic level
+     * @var boolean
+     */
+    private $_automatic = true;
 
-    private          $_automatic = true;
+    /**
+     * Internal cache for all adapters
+     * @var Zend_Cache_Core
+     */
     protected static $_cache     = null;
 
-    // Scan options
+    /**
+     * Scans for the locale within the name of the directory
+     * @constant integer
+     */
     const LOCALE_DIRECTORY = 1;
+
+    /**
+     * Scans for the locale within the name of the file
+     * @constant integer
+     */
     const LOCALE_FILENAME  = 2;
 
     /**
      * Array with all options, each adapter can have own additional options
-     *
+     *       'clear'  => clears already loaded data when adding new files
+     *       'scan'   => searches for translation files using the LOCALE constants
+     *       'locale' => the actual set locale to use
      * @var array
      */
     protected $_options = array(
-        'clear'  => false, // clear previous loaded translation data
-        'scan'   => null,  // where to find the locale
-        'locale' => 'auto' // actual set locale/language 
+        'clear'  => false,
+        'scan'   => null,
+        'locale' => 'auto' 
     );
 
     /**
      * Translation table
-     *
      * @var array
      */
     protected $_translate = array();
 
-
     /**
      * Generates the adapter
      *
-     * @param  string|array        $data      Translation data for this adapter
-     * @param  string|Zend_Locale  $locale    OPTIONAL Locale/Language to set, identical with Locale identifiers
-     *                                        see Zend_Locale for more information
-     * @param  string|array        $options   Options for the adaptor
+     * @param  string|array       $data    Translation data or filename for this adapter
+     * @param  string|Zend_Locale $locale  (optional) Locale/Language to set, identical with Locale
+     *                                     identifiers see Zend_Locale for more information
+     * @param  array              $options (optional) Options for the adaptor
      * @throws Zend_Translate_Exception
+     * @return void
      */
     public function __construct($data, $locale = null, array $options = array())
     {
@@ -72,12 +94,18 @@ abstract class Zend_Translate_Adapter {
             $id = 'Zend_Translate_' . $this->toString();
             if ($result = self::$_cache->load($id)) {
                 $this->_translate = unserialize($result);
-                return true;
+                $this->_options   = $this->_translate['_options_'];
+                unset($this->_translate['_options_']);
+                return;
             }
         }
 
+        if (($locale === "auto") or ($locale === null)) {
+            $this->_automatic = true;
+        } else {
+            $this->_automatic = false;
+        }
         $this->addTranslation($data, $locale, $options);
-        $this->_automatic = true;
     }
 
     /**
@@ -87,12 +115,12 @@ abstract class Zend_Translate_Adapter {
      * If $clear parameter is true, then translation data for specified
      * language is replaced and added otherwise
      *
-     * @param  array|string          $data    Translation data
-     * @param  string|Zend_Locale    $locale  Locale/Language to add data for, identical with locale identifier,
-     *                                        see Zend_Locale for more information
-     * @param  array                 $options OPTIONAL Option for this Adapter
+     * @param  array|string       $data    Translation data
+     * @param  string|Zend_Locale $locale  (optional) Locale/Language to add data for, identical
+     *                                        with locale identifier, see Zend_Locale for more information
+     * @param  array              $options (optional) Option for this Adapter
      * @throws Zend_Translate_Exception
-     * @return Zend_Translate_Adapter
+     * @return Zend_Translate_Adapter Provides a fluid interface
      */
     public function addTranslation($data, $locale = null, array $options = array())
     {
@@ -106,46 +134,55 @@ abstract class Zend_Translate_Adapter {
 
         $this->setOptions($options);
         if (is_string($data) and is_dir($data)) {
+            $prev  = '';
             foreach (new RecursiveIteratorIterator(
                      new RecursiveDirectoryIterator($data, RecursiveDirectoryIterator::KEY_AS_PATHNAME), 
                      RecursiveIteratorIterator::SELF_FIRST) as $file => $info) {
+                $file = $info->getFilename();
                 if ($info->isDir()) {
-
-                    $directory = $info->getPath();
                     // pathname as locale
-                    if (($this->_options['scan'] === self::LOCALE_DIRECTORY) and (Zend_Locale::isLocale((string) $info))) {
-                        $locale = (string) $info;
+                    if (($this->_options['scan'] === self::LOCALE_DIRECTORY) and (Zend_Locale::isLocale($file))) {
+                        if (strlen($prev) <= strlen($file)) {
+                            $locale = $file;
+                            $prev   = $locale;
+                        }
                     }
-
                 } else if ($info->isFile()) {
-
                     // filename as locale
                     if ($this->_options['scan'] === self::LOCALE_FILENAME) {
-                        $filename = explode('.', (string) $info);
+                        $filename = explode('.', $file);
                         array_pop($filename);
                         $filename = implode('.', $filename);
-                        if (Zend_Locale::isLocale($filename)) {
+                        if (Zend_Locale::isLocale((string) $filename)) {
                             $locale = (string) $filename;
                         } else {
                             $found = false;
-                            $parts = explode('.', $filename);
+                            $parts  = explode('.', $filename);
+                            $parts2 = array();
                             foreach($parts as $token) {
-                                $parts = array_merge(explode('_', $token), $parts);
+                                $parts2 += explode('_', $token);
                             }
+                            $parts  = array_merge($parts, $parts2);
+                            $parts2 = array();
                             foreach($parts as $token) {
-                                $parts = array_merge(explode('-', $token), $parts);
+                                $parts2 += explode('-', $token);
                             }
+                            $parts = array_merge($parts, $parts2);
                             $parts = array_unique($parts);
+                            $prev  = '';
                             foreach($parts as $token) {
                                 if (Zend_Locale::isLocale($token)) {
-                                    $locale = $token;
+                                    if (strlen($prev) <= strlen($token)) {
+                                        $locale = $token;
+                                        $prev   = $token;
+                                    }
                                 }
                             }
                         }
                     }
                     try {
-                        $this->_addTranslationData((string) $info->getPathname(), $locale, $this->_options);
-                        if ((array_key_exists($locale, $this->_translate)) and (count($this->_translate[$locale]) > 0)) {
+                        $this->_addTranslationData($info->getPathname(), $locale, $this->_options);
+                        if ((isset($this->_translate[$locale]) === true) and (count($this->_translate[$locale]) > 0)) {
                             $this->setLocale($locale);
                         }
                     } catch (Zend_Translate_Exception $e) {
@@ -155,23 +192,22 @@ abstract class Zend_Translate_Adapter {
             }
         } else {
             $this->_addTranslationData($data, $locale, $this->_options);
-            if ((array_key_exists($locale, $this->_translate)) and (count($this->_translate[$locale]) > 0)) {
+            if ((isset($this->_translate[$locale]) === true) and (count($this->_translate[$locale]) > 0)) {
                 $this->setLocale($locale);
             }
         }
-        if ((array_key_exists($originate, $this->_translate)) and (count($this->_translate[$originate]) > 0)) {
+        if ((isset($translate[$originate]) === true) and (count($this->_translate[$originate]) > 0)) {
             $this->setLocale($originate);
         }
         return $this;
     }
 
-
     /**
      * Sets new adapter options
      *
-     * @param  array  $options  Adapter options
+     * @param  array $options Adapter options
      * @throws Zend_Translate_Exception
-     * @return Zend_Translate_Adapter
+     * @return Zend_Translate_Adapter Provides a fluid interface
      */
     public function setOptions(array $options = array())
     {
@@ -188,22 +224,22 @@ abstract class Zend_Translate_Adapter {
     /**
      * Returns the adapters name and it's options
      *
-     * @param  string|null  $optionKey  String returns this option
-     *                                  null returns all options
-     * @return integer|string|array
+     * @param  string|null $optionKey String returns this option
+     *                                null returns all options
+     * @return integer|string|array|null
      */
     public function getOptions($optionKey = null)
     {
         if ($optionKey === null) {
             return $this->_options;
         }
-        if (array_key_exists(strtolower($optionKey), $this->_options)) {
-            return $this->_options[strtolower($optionKey)];
+        $optionKey = strtolower($optionKey);
+        if (isset($this->_options[$optionKey]) === true) {
+            return $this->_options[$optionKey];
         }
         return null;
     }
 
-
     /**
      * Gets locale
      *
@@ -214,26 +250,31 @@ abstract class Zend_Translate_Adapter {
         return $this->_options['locale'];
     }
 
-
     /**
      * Sets locale
      *
-     * @param  string|Zend_Locale  $locale  Locale to set
+     * @param  string|Zend_Locale $locale Locale to set
      * @throws Zend_Translate_Exception
-     * @return Zend_Translate_Adapter
+     * @return Zend_Translate_Adapter Provides a fluid interface
      */
     public function setLocale($locale)
     {
         if ($locale instanceof Zend_Locale) {
             $locale = $locale->toString();
         } else if (!$locale = Zend_Locale::isLocale($locale)) {
+            /**
+             * @see Zend_Translate_Exception
+             */
             require_once 'Zend/Translate/Exception.php';
             throw new Zend_Translate_Exception("The given Language ({$locale}) does not exist");
         }
 
-        if (!array_key_exists($locale, $this->_translate) and empty($this->_translate[$locale])) {
+        if (empty($this->_translate[$locale]) === true) {
             $temp = explode('_', $locale);
-            if (!array_key_exists($temp[0], $this->_translate)) {
+            if (isset($this->_translate[$temp[0]]) === false) {
+                /**
+                 * @see Zend_Translate_Exception
+                 */
                 require_once 'Zend/Translate/Exception.php';
                 throw new Zend_Translate_Exception("Language ({$locale}) has to be added before it can be used.");
             }
@@ -241,15 +282,15 @@ abstract class Zend_Translate_Adapter {
         }
 
         $this->_options['locale'] = $locale;
-        if ($locale == "auto") {
+        if ($locale === "auto") {
             $this->_automatic = true;
         } else {
             $this->_automatic = false;
         }
+
         return $this;
     }
 
-
     /**
      * Returns the available languages from this adapter
      *
@@ -267,12 +308,11 @@ abstract class Zend_Translate_Adapter {
         return $result;
     }
 
-
     /**
      * Returns all available message ids from this adapter
      * If no locale is given, the actual language will be used
      *
-     * @param  $locale  String|Zend_Locale  Language to return the message ids from
+     * @param  string|Zend_Locale $locale (optional) Language to return the message ids from
      * @return array
      */
     public function getMessageIds($locale = null)
@@ -280,52 +320,52 @@ abstract class Zend_Translate_Adapter {
         if (empty($locale) or !$this->isAvailable($locale)) {
             $locale = $this->_options['locale'];
         }
-        return array_keys($this->_translate[$locale]);
-    }
 
+        return array_keys($this->_translate[(string) $locale]);
+    }
 
     /**
      * Returns all available translations from this adapter
      * If no locale is given, the actual language will be used
      * If 'all' is given the complete translation dictionary will be returned
      *
-     * @param  $locale  String|Zend_Locale  Language to return the messages from
+     * @param  string|Zend_Locale $locale (optional) Language to return the messages from
      * @return array
      */
     public function getMessages($locale = null)
     {
-        if ($locale == 'all') {
+        if ($locale === 'all') {
             return $this->_translate;
         }
-        if (empty($locale) or !$this->isAvailable($locale)) {
+
+        if ((empty($locale) === true) or ($this->isAvailable($locale) === false)) {
             $locale = $this->_options['locale'];
         }
-        return $this->_translate[$locale];
-    }
 
+        return $this->_translate[(string) $locale];
+    }
 
     /**
      * Is the wished language available ?
      *
-     * @param  string|Zend_Locale  $locale  Language to search for, identical with locale identifier,
-     *                                      see Zend_Locale for more information
+     * @see    Zend_Locale
+     * @param  string|Zend_Locale $locale Language to search for, identical with locale identifier,
+     *                                    @see Zend_Locale for more information
      * @return boolean
      */
     public function isAvailable($locale)
     {
-        if ($locale instanceof Zend_Locale) {
-            $locale = $locale->toString();
-        }
-
-        return array_key_exists($locale, $this->_translate);
+        $return = isset($this->_translate[(string) $locale]);
+        return $return;
     }
 
     /**
      * Load translation data
      *
-     * @param  mixed               $data
-     * @param  string|Zend_Locale  $locale
-     * @param  array               $options
+     * @param  mixed              $data
+     * @param  string|Zend_Locale $locale
+     * @param  array              $options (optional)
+     * @return void
      */
     abstract protected function _loadTranslationData($data, $locale, array $options = array());
 
@@ -336,31 +376,35 @@ abstract class Zend_Translate_Adapter {
      * If $clear parameter is true, then translation data for specified
      * language is replaced and added otherwise
      *
-     * @param  array|string          $data    Translation data
-     * @param  string|Zend_Locale    $locale  Locale/Language to add data for, identical with locale identifier,
-     *                                        see Zend_Locale for more information
-     * @param  array                 $options OPTIONAL Option for this Adapter
+     * @see    Zend_Locale
+     * @param  array|string       $data    Translation data
+     * @param  string|Zend_Locale $locale  Locale/Language to add data for, identical with locale identifier,
+     *                                     @see Zend_Locale for more information
+     * @param  array              $options (optional) Option for this Adapter
      * @throws Zend_Translate_Exception
-     * @return Zend_Translate_Adapter
+     * @return Zend_Translate_Adapter Provides a fluid interface
      */
     private function _addTranslationData($data, $locale, array $options = array())
     {
         if (!$locale = Zend_Locale::isLocale($locale)) {
+            /**
+             * @see Zend_Translate_Exception
+             */
             require_once 'Zend/Translate/Exception.php';
             throw new Zend_Translate_Exception("The given Language ({$locale}) does not exist");
         }
 
-        if (!array_key_exists($locale, $this->_translate)) {
+        if (isset($this->_translate[$locale]) === false) {
             $this->_translate[$locale] = array();
         }
 
         $this->_loadTranslationData($data, $locale, $options);
         if ($this->_automatic === true) {
             $find = new Zend_Locale($locale);
-            $browser = $find->getBrowser() + $find->getEnvironment();
+            $browser = $find->getEnvironment() + $find->getBrowser();
             arsort($browser);
             foreach($browser as $language => $quality) {
-                if (array_key_exists($language, $this->_translate)) {
+                if (isset($this->_translate[$language]) === true) {
                     $this->_options['locale'] = $language;
                     break;
                 }
@@ -369,19 +413,21 @@ abstract class Zend_Translate_Adapter {
 
         if (isset(self::$_cache)) {
             $id = 'Zend_Translate_' . $this->toString();
-            self::$_cache->save( serialize($this->_translate), $id);
+            $temp = $this->_translate;
+            $temp['_options_'] = $this->_options;
+            self::$_cache->save( serialize($temp), $id);
         }
         return $this;
     }
 
-
     /**
      * Translates the given string
      * returns the translation
      *
-     * @param  string              $messageId  Translation string
-     * @param  string|Zend_Locale  $locale     OPTIONAL Locale/Language to use, identical with locale identifier,
-     *                                         see Zend_Locale for more information
+     * @see Zend_Locale
+     * @param  string             $messageId Translation string
+     * @param  string|Zend_Locale $locale    (optional) Locale/Language to use, identical with
+     *                                       locale identifier, @see Zend_Locale for more information
      * @return string
      */
     public function translate($messageId, $locale = null)
@@ -394,16 +440,14 @@ abstract class Zend_Translate_Adapter {
             return $messageId;
         }
 
-        if ((is_array($this->_translate) and array_key_exists($locale, $this->_translate)) and
-            (is_array($this->_translate[$locale]) and array_key_exists($messageId, $this->_translate[$locale]))) {
+        if (isset($this->_translate[$locale][$messageId]) === true) {
             // return original translation
             return $this->_translate[$locale][$messageId];
         } else if (strlen($locale) != 2) {
             // faster than creating a new locale and separate the leading part
             $locale = substr($locale, 0, -strlen(strrchr($locale, '_')));
 
-            if ((is_array($this->_translate) and array_key_exists($locale, $this->_translate)) and
-                (is_array($this->_translate[$locale]) and array_key_exists($messageId, $this->_translate[$locale]))) {
+            if (isset($this->_translate[$locale][$messageId]) === true) {
                 // return regionless translation (en_US -> en)
                 return $this->_translate[$locale][$messageId];
             }
@@ -413,14 +457,13 @@ abstract class Zend_Translate_Adapter {
         return $messageId;
     }
 
-
     /**
      * Translates the given string
      * returns the translation
      *
-     * @param  string              $messageId  Translation string
-     * @param  string|Zend_Locale  $locale     OPTIONAL Locale/Language to use, identical with locale identifier,
-     *                                         see Zend_Locale for more information
+     * @param  string             $messageId Translation string
+     * @param  string|Zend_Locale $locale    (optional) Locale/Language to use, identical with locale
+     *                                       identifier, @see Zend_Locale for more information
      * @return string
      */
     public function _($messageId, $locale = null)
@@ -428,17 +471,16 @@ abstract class Zend_Translate_Adapter {
         return $this->translate($messageId, $locale);
     }
 
-
     /**
      * Checks if a string is translated within the source or not
      * returns boolean
      *
-     * @param  string              $messageId  Translation string
-     * @param  boolean             $original   OPTIONAL Allow translation only for original language
-     *                                         when true, a translation for 'en_US' would give false when it can
-     *                                         be translated with 'en' only
-     * @param  string|Zend_Locale  $locale     OPTIONAL Locale/Language to use, identical with locale identifier,
-     *                                         see Zend_Locale for more information
+     * @param  string             $messageId Translation string
+     * @param  boolean            $original  (optional) Allow translation only for original language
+     *                                       when true, a translation for 'en_US' would give false when it can
+     *                                       be translated with 'en' only
+     * @param  string|Zend_Locale $locale    (optional) Locale/Language to use, identical with locale identifier,
+     *                                       see Zend_Locale for more information
      * @return boolean
      */
     public function isTranslated($messageId, $original = false, $locale = null)
@@ -456,26 +498,23 @@ abstract class Zend_Translate_Adapter {
             }
         }
 
-        if ((is_array($this->_translate) and array_key_exists($locale, $this->_translate)) and
-            (is_array($this->_translate[$locale]) and array_key_exists($messageId, $this->_translate[$locale]))) {
+        if (isset($this->_translate[$locale][$messageId]) === true) {
             // return original translation
             return true;
         } else if ((strlen($locale) != 2) and ($original === false)) {
             // faster than creating a new locale and separate the leading part
             $locale = substr($locale, 0, -strlen(strrchr($locale, '_')));
 
-            if ((is_array($this->_translate) and array_key_exists($locale, $this->_translate)) and
-                (is_array($this->_translate[$locale]) and array_key_exists($messageId, $this->_translate[$locale]))) {
+            if (isset($this->_translate[$locale][$messageId]) === true) {
                 // return regionless translation (en_US -> en)
                 return true;
             }
         }
 
-        // no translation found, return original
+        // No translation found, return original
         return false;
     }
 
-
     /**
      * Sets a cache for all Zend_Translate_Adapters
      *
@@ -486,6 +525,15 @@ abstract class Zend_Translate_Adapter {
         self::$_cache = $cache;
     }
 
+    /**
+     * Returns the set cache
+     *
+     * @return Zend_Cache_Core The set cache
+     */
+    public static function getCache()
+    {
+        return self::$_cache;
+    }
 
     /**
      * Returns the adapter name
@@ -493,4 +541,4 @@ abstract class Zend_Translate_Adapter {
      * @return string
      */
     abstract public function toString();
-}
+}
\ No newline at end of file
