@@ -183,6 +183,8 @@ class Zend_Form_DisplayGroupTest extends PHPUnit_Framework_TestCase
 
     public function testDefaultDecoratorsRegistered()
     {
+        $this->_checkZf2794();
+
         $decorator = $this->group->getDecorator('FormElements');
         $this->assertTrue($decorator instanceof Zend_Form_Decorator_FormElements);
         $decorator = $this->group->getDecorator('Fieldset');
@@ -212,6 +214,8 @@ class Zend_Form_DisplayGroupTest extends PHPUnit_Framework_TestCase
 
     public function testCanAddSingleDecoratorAsString()
     {
+        $this->_checkZf2794();
+
         $this->group->clearDecorators();
         $this->assertFalse($this->group->getDecorator('form'));
 
@@ -239,6 +243,8 @@ class Zend_Form_DisplayGroupTest extends PHPUnit_Framework_TestCase
 
     public function testCanRetrieveSingleDecoratorRegisteredAsDecoratorObjectUsingShortName()
     {
+        $this->_checkZf2794();
+
         $this->group->clearDecorators();
         $this->assertFalse($this->group->getDecorator('form'));
 
@@ -250,6 +256,8 @@ class Zend_Form_DisplayGroupTest extends PHPUnit_Framework_TestCase
 
     public function testCanAddMultipleDecorators()
     {
+        $this->_checkZf2794();
+
         $this->group->clearDecorators();
         $this->assertFalse($this->group->getDecorator('form'));
 
@@ -267,13 +275,38 @@ class Zend_Form_DisplayGroupTest extends PHPUnit_Framework_TestCase
 
     public function testCanRemoveDecorator()
     {
+        $this->_checkZf2794();
+
         $this->testDefaultDecoratorsRegistered();
         $this->group->removeDecorator('form');
         $this->assertFalse($this->group->getDecorator('form'));
     }
 
+    /**
+     * @see ZF-3069
+     */
+    public function testRemovingNamedDecoratorsShouldWork()
+    {
+        $this->_checkZf2794();
+        $this->group->setDecorators(array(
+            'FormElements',
+            array(array('div' => 'HtmlTag'), array('tag' => 'div')),
+            array(array('div2' => 'HtmlTag'), array('tag' => 'div')),
+        ));
+        $decorators = $this->group->getDecorators();
+        $this->assertTrue(array_key_exists('div', $decorators));
+        $this->assertTrue(array_key_exists('div2', $decorators));
+        $this->group->removeDecorator('div');
+        $decorators = $this->group->getDecorators();
+        $this->assertFalse(array_key_exists('div', $decorators));
+        $this->assertTrue(array_key_exists('div2', $decorators));
+    }
+
+
     public function testCanClearAllDecorators()
     {
+        $this->_checkZf2794();
+
         $this->testCanAddMultipleDecorators();
         $this->group->clearDecorators();
         $this->assertFalse($this->group->getDecorator('viewHelper'));
@@ -282,6 +315,8 @@ class Zend_Form_DisplayGroupTest extends PHPUnit_Framework_TestCase
 
     public function testCanAddDecoratorAliasesToAllowMultipleDecoratorsOfSameType()
     {
+        $this->_checkZf2794();
+
         $this->group->setDecorators(array(
             array('HtmlTag', array('tag' => 'fieldset')),
             array('decorator' => array('FooBar' => 'HtmlTag'), 'options' => array('tag' => 'dd')),
@@ -295,6 +330,39 @@ class Zend_Form_DisplayGroupTest extends PHPUnit_Framework_TestCase
         $this->assertEquals('fieldset', $decorator->getOption('tag'));
     }
 
+    /**
+     * @see ZF-3494
+     */
+    public function testGetViewShouldNotReturnNullWhenViewRendererIsActive()
+    {
+        require_once 'Zend/Controller/Action/HelperBroker.php';
+        $viewRenderer = Zend_Controller_Action_HelperBroker::getStaticHelper('ViewRenderer');
+        $viewRenderer->initView();
+        $view = $this->group->getView();
+        $this->assertSame($viewRenderer->view, $view);
+    }
+
+    public function testRetrievingNamedDecoratorShouldNotReorderDecorators()
+    {
+        $this->group->setDecorators(array(
+            'FormElements',
+            array(array('dl' => 'HtmlTag'), array('tag' => 'dl')),
+            array(array('div' => 'HtmlTag'), array('tag' => 'div')),
+            array(array('fieldset' => 'HtmlTag'), array('tag' => 'fieldset')),
+        ));
+
+        $decorator  = $this->group->getDecorator('div');
+        $decorators = $this->group->getDecorators();
+        $i          = 0;
+        $order      = array();
+
+        foreach (array_keys($decorators) as $name) {
+            $order[$name] = $i;
+            ++$i;
+        }
+        $this->assertEquals(2, $order['div'], var_export($order, 1));
+    }
+
     public function testRenderingRendersAllElementsWithinFieldsetByDefault()
     {
         $foo  = new Zend_Form_Element_Text('foo');
@@ -302,7 +370,7 @@ class Zend_Form_DisplayGroupTest extends PHPUnit_Framework_TestCase
 
         $this->group->addElements(array($foo, $bar));
         $html = $this->group->render($this->getView());
-        $this->assertRegexp('#^<dt></dt><dd><fieldset.*?</fieldset></dd>$#s', $html, $html);
+        $this->assertRegexp('#^<dt>&nbsp;</dt><dd><fieldset.*?</fieldset></dd>$#s', $html, $html);
         $this->assertContains('<input', $html, $html);
         $this->assertContains('"foo"', $html);
         $this->assertContains('"bar"', $html);
@@ -316,7 +384,7 @@ class Zend_Form_DisplayGroupTest extends PHPUnit_Framework_TestCase
         $this->group->addElements(array($foo, $bar))
                     ->setView($this->getView());
         $html = $this->group->__toString();
-        $this->assertRegexp('#^<dt></dt><dd><fieldset.*?</fieldset></dd>$#s', $html, $html);
+        $this->assertRegexp('#^<dt>&nbsp;</dt><dd><fieldset.*?</fieldset></dd>$#s', $html, $html);
         $this->assertContains('<input', $html);
         $this->assertContains('"foo"', $html);
         $this->assertContains('"bar"', $html);
@@ -336,7 +404,7 @@ class Zend_Form_DisplayGroupTest extends PHPUnit_Framework_TestCase
     {
         $this->group->setDecorators(array(
             array(
-                'decorator' => 'Callback', 
+                'decorator' => 'Callback',
                 'options'   => array('callback' => array($this, 'raiseDecoratorException'))
             ),
         ));
@@ -389,7 +457,7 @@ class Zend_Form_DisplayGroupTest extends PHPUnit_Framework_TestCase
         $baz = new Zend_Form_Element('baz');
         $this->group->addElements(array($foo, $bar, $baz));
     }
-    
+
     public function testDisplayGroupIsIterableAndIteratesElements()
     {
         $this->setupIteratorElements();
@@ -415,6 +483,29 @@ class Zend_Form_DisplayGroupTest extends PHPUnit_Framework_TestCase
         $this->assertSame($expected, $received);
     }
 
+    public function testDisplayGroupIteratesElementsInExpectedOrderWhenFirstElementHasNoOrderSpecified()
+    {
+        $a = new Zend_Form_Element('a',array('label'=>'a'));
+        $b = new Zend_Form_Element('b',array('label'=>'b', 'order' => 0));
+        $c = new Zend_Form_Element('c',array('label'=>'c', 'order' => 1));
+        $this->group->addElement($a)
+                    ->addElement($b)
+                    ->addElement($c)
+                    ->setView($this->getView());
+        $test = $this->group->render();
+        $this->assertContains('name="a"', $test);
+        if (!preg_match_all('/(<input[^>]+>)/', $test, $matches)) {
+            $this->fail('Expected markup not found');
+        }
+        $order = array();
+        foreach ($matches[1] as $element) {
+            if (preg_match('/name="(a|b|c)"/', $element, $m)) {
+                $order[] = $m[1];
+            }
+        }
+        $this->assertSame(array('b', 'c', 'a'), $order);
+    }
+
     public function testRemovingElementsShouldNotRaiseExceptionsDuringIteration()
     {
         $this->setupIteratorElements();
@@ -462,8 +553,9 @@ class Zend_Form_DisplayGroupTest extends PHPUnit_Framework_TestCase
     public function testSetOptionsOmitsAccessorsRequiringObjectsOrMultipleParams()
     {
         $options = $this->getOptions();
-        $options['config']       = new Zend_Config($options);
-        $options['options']      = $options;
+        $config                  = new Zend_Config($options);
+        $options['config']       = $config;
+        $options['options']      = $config->toArray();
         $options['pluginLoader'] = true;
         $options['view']         = true;
         $options['translator']   = true;
@@ -473,6 +565,8 @@ class Zend_Form_DisplayGroupTest extends PHPUnit_Framework_TestCase
 
     public function testSetOptionsSetsArrayOfStringDecorators()
     {
+        $this->_checkZf2794();
+
         $options = $this->getOptions();
         $options['decorators'] = array('label', 'form');
         $this->group->setOptions($options);
@@ -486,6 +580,8 @@ class Zend_Form_DisplayGroupTest extends PHPUnit_Framework_TestCase
 
     public function testSetOptionsSetsArrayOfArrayDecorators()
     {
+        $this->_checkZf2794();
+
         $options = $this->getOptions();
         $options['decorators'] = array(
             array('label', array('id' => 'mylabel')),
@@ -507,15 +603,17 @@ class Zend_Form_DisplayGroupTest extends PHPUnit_Framework_TestCase
 
     public function testSetOptionsSetsArrayOfAssocArrayDecorators()
     {
+        $this->_checkZf2794();
+
         $options = $this->getOptions();
         $options['decorators'] = array(
             array(
                 'options'   => array('id' => 'mylabel'),
-                'decorator' => 'label', 
+                'decorator' => 'label',
             ),
             array(
                 'options'   => array('id' => 'form'),
-                'decorator' => 'form', 
+                'decorator' => 'form',
             ),
         );
         $this->group->setOptions($options);
@@ -604,6 +702,19 @@ class Zend_Form_DisplayGroupTest extends PHPUnit_Framework_TestCase
         $decorators = $group->getDecorators();
         $this->assertTrue(empty($decorators));
     }
+
+    /**
+     * Used by test methods susceptible to ZF-2794, marks a test as incomplete
+     *
+     * @link   http://framework.zend.com/issues/browse/ZF-2794
+     * @return void
+     */
+    protected function _checkZf2794()
+    {
+        if (strtolower(substr(PHP_OS, 0, 3)) == 'win' && version_compare(PHP_VERSION, '5.1.4', '=')) {
+            $this->markTestIncomplete('Error occurs for PHP 5.1.4 on Windows');
+        }
+    }
 }
 
 class Zend_Form_DisplayGroupTest_DisplayGroup extends Zend_Form_DisplayGroup
