@@ -16,7 +16,7 @@
  * @package    Zend_OpenId
  * @copyright  Copyright (c) 2005-2008 Zend Technologies USA Inc. (http://www.zend.com)
  * @license    http://framework.zend.com/license/new-bsd     New BSD License
- * @version    $Id: ConsumerTest.php 8725 2008-03-10 11:12:47Z alexander $
+ * @version    $Id: ConsumerTest.php 9910 2008-07-02 22:40:29Z shahar $
  */
 
 /**
@@ -330,7 +330,8 @@ class Zend_OpenId_ConsumerTest extends PHPUnit_Framework_TestCase
                            "Connection: close\r\n" .
                            "Accept-encoding: gzip, deflate\r\n" .
                            "Content-type: application/x-www-form-urlencoded\r\n" .
-                           "User-agent: Zend_OpenId\r\n\r\n",
+                           "User-agent: Zend_OpenId\r\n" . 
+                           "Content-length: 0\r\n\r\n",
                            $http->getLastRequest() );
 
         // Test GET request with parameters
@@ -446,7 +447,7 @@ class Zend_OpenId_ConsumerTest extends PHPUnit_Framework_TestCase
             $this->assertTrue( $storage->getAssociation(self::SERVER, $handle, $macFunc, $secret, $expires) );
             $this->assertSame( "0123456789absdef0123456789absdef", $handle );
             $this->assertSame( "sha1", $macFunc );
-            $this->assertSame( "8382aea922560ece833ba55fa53b7a975f597370", bin2hex($secret) );
+            $this->assertSame( "e36624c686748f6b646648f12748ffd157e4d4dd", bin2hex($secret) );
             $this->assertTrue( $storage->delAssociation(self::SERVER) );
 
             // Wrong OpenID 2.0 association response (wrong ns)
@@ -717,6 +718,19 @@ class Zend_OpenId_ConsumerTest extends PHPUnit_Framework_TestCase
                            "</head><body</body></html>\n");
         $id = self::ID;
         $this->assertFalse( $consumer->discovery($id, $server, $version) );
+
+        // Test HTML based discovery with multivalue rel (OpenID 1.1)
+        $storage->delDiscoveryInfo(self::ID);
+        $test->setResponse("HTTP/1.1 200 OK\r\n\r\n" .
+                           "<html><head>\n" .
+                           "<link rel=\" aaa openid.server bbb \" href=\"" . self::SERVER . "\">\n" .
+                           "<link rel=\"aaa openid.delegate\" href=\"" . self::REAL_ID . "\">\n" .
+                           "</head><body</body></html>\n");
+        $id = self::ID;
+        $this->assertTrue( $consumer->discovery($id, $server, $version) );
+        $this->assertSame( self::REAL_ID, $id );
+        $this->assertSame( self::SERVER, $server );
+        $this->assertSame( 1.1, $version );
     }
 
     /**
@@ -752,9 +766,14 @@ class Zend_OpenId_ConsumerTest extends PHPUnit_Framework_TestCase
         $storage->purgeNonces();
         $this->assertTrue( $consumer->verify($params) );
 
+        $storage->delDiscoveryInfo(self::ID);
+        $storage->addDiscoveryInfo(self::ID, self::REAL_ID, self::SERVER, 2.0, $expiresIn);
+
         // HMAC-SHA256
         $consumer->clearAssociation();
         $params = array(
+            "openid_ns" => Zend_OpenId::NS_2_0,
+            "openid_op_endpoint" => self::SERVER,
             "openid_return_to" => "http://www.zf-test.com/test.php",
             "openid_assoc_handle" => self::HANDLE,
             "openid_claimed_id" => self::ID,
@@ -772,6 +791,8 @@ class Zend_OpenId_ConsumerTest extends PHPUnit_Framework_TestCase
         // HMAC-SHA256 (duplicate response_nonce)
         $consumer->clearAssociation();
         $params = array(
+            "openid_ns" => Zend_OpenId::NS_2_0,
+            "openid_op_endpoint" => self::SERVER,
             "openid_return_to" => "http://www.zf-test.com/test.php",
             "openid_assoc_handle" => self::HANDLE,
             "openid_claimed_id" => self::ID,
@@ -785,6 +806,9 @@ class Zend_OpenId_ConsumerTest extends PHPUnit_Framework_TestCase
         $storage->addAssociation(self::SERVER, self::HANDLE, "sha256", pack("H*", "ed901bc561c29fd7bb42862e5f09fa37e7944a7ee72142322f34a21bfe1384b8"), $expiresIn);
         $this->assertFalse( $consumer->verify($params) );
 
+        $storage->delDiscoveryInfo(self::ID);
+        $storage->addDiscoveryInfo(self::ID, self::REAL_ID, self::SERVER, 1.1, $expiresIn);
+
         // wrong signature
         $consumer->clearAssociation();
         $params = array(
